"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.objectToQueryParameter = exports.isBinaryData = exports.hasBinaryData = exports.processData = undefined;

var _urlParse = require("url-parse");

var _urlParse2 = _interopRequireDefault(_urlParse);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { return obj && typeof Symbol !== "undefined" && obj.constructor === Symbol ? "symbol" : typeof obj; }

function processData(inputs, ajaxOption) {
    var formData = new FromData();

    if (ajaxOption.processData === false) {
        return inputs;
    }

    if (typeof inputs === "string") {
        return inputs;
    }

    if (inputs instanceof FromData) {
        // FormData.getAll is now not available in some browsers (or flag required)
        // This is untouchable.
        // (https://developer.mozilla.org/en-US/docs/Web/API/FormData)
        return inputs;
    }

    if ((typeof inputs === "undefined" ? "undefined" : _typeof(inputs)) === "object") {
        Object.keys(inputs).forEach(function (name) {
            var value = inputs[name];

            if (Array.isArray(value)) {
                value.forEach(function (value) {
                    formData.append(name, value);
                });
                return;
            }

            formData.append(name, inputs[name]);
        });
    }

    return formData;
}

function hasBinaryData(values) {
    var hasBinary = false;

    if (Array.isArray(values)) {
        Array.forEach(function (data) {
            hasBinary = hasBinary || isBinaryData(data);
        });
    } else if ((typeof values === "undefined" ? "undefined" : _typeof(values)) === "object") {
        Object.keys(values).forEach(function (key) {
            var value = values[key];
            hasBinary = hasBinary || isBinaryData(value) || hasBinaryData(value);
        });
    }

    return hasBinary;
}

function isBinaryData(data) {
    return data instanceof Blob || data instanceof File || data instanceof ArrayBuffer;
}

function queryToObject(string) {
    var url = (0, _urlParse2.default)(string);
    return url.query;
}

function objectToQueryParameter(obj) {
    if (typeof obj === "string") {
        return obj;
    }

    var params = [];

    Object.keys(obj).forEach(function (key) {
        var value = obj[key];
        params.push([encodeURIComponent(key), "=", encodeURIComponent(value)].join(""));
    });

    return params.join("&");
}

exports.processData = processData;
exports.hasBinaryData = hasBinaryData;
exports.isBinaryData = isBinaryData;
exports.objectToQueryParameter = objectToQueryParameter;