"use strict";

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

// Check valid maya.js project
function _isValidMayaProject(cwd) {
    try {
        var packageJson = require(_path2.default.join(cwd, "package.json"));
        var deps = packageJson.dependencies;
        var devDeps = packageJson.devDependencies;

        if (deps && Object.keys(deps).indexOf("maya") !== -1) {
            return true;
        }

        if (devDeps && Object.keys(devDeps).indexOf("maya") !== -1) {
            return true;
        }

        return "DEPENDENCY_NOT_FOUND";
    } catch (e) {
        return "PACKAGE_JSON_NOT_FOUND";
    }
};

function _exitIfInvalidMayaProject(cwd) {
    var valid = _isValidMayaProject(cwd);

    if (valid === true) {
        return;
    }

    switch (valid) {
        case "DEPENDENCY_NOT_FOUND":
            console.error("\u001b[31mHere is not maya.js project. \n(reliance on maya.js can not be found in `dependencies` or `devDependencies`)\u001b[m");
            break;

        case "PACKAGE_JSON_NOT_FOUND":
            console.error("\u001b[31mHere is not maya.js project. \n(package.json not found.)\u001b[m");
            break;
    }

    process.exit(-1);
}

module.exports._isValidMayaProject = _isValidMayaProject;
module.exports._exitIfInvalidMayaProject = _exitIfInvalidMayaProject;

module.exports.run = function run() {
    var cwd = process.cwd();

    var _process$argv = _toArray(process.argv);

    var command = _process$argv[2];

    var args = _process$argv.slice(3);

    switch (command) {
        case "init":
            require("./init").run(args);
            break;

        case "cqc":
            _exitIfInvalidMayaProject(cwd);
            require("./cqc")(args);
            break;

        case "_dev":
            process.chdir(_path2.default.join(__dirname, "../../devapp"));
            require("./cqc")(args);
            break;

        case "export":
            _exitIfInvalidMayaProject(cwd);
            require("./export")(args);
            break;

        case "g":
        case "generate":
            _exitIfInvalidMayaProject(cwd);
            require("./generate")(args);
            break;

        case "h":
        case "help":
        default:
            if (command !== undefined) {
                console.log("\u001b[31mCommand not found : " + command + "\u001b[m\n");
            }
            console.log("Usage :\n    maya <command> [<options>...]\n\nCommands :\n    cqc\t\t\tStart maya.js server.\n    export\t\tExport maya.js components for other libraries.\n    generate(g)\t\tGenerate some components\n    help\t\tShow this help\n");
            process.exit(0);
    }
};