"use strict";

var _slicedToArray = (function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; })();

var _lodash = require("lodash");

var _lodash2 = _interopRequireDefault(_lodash);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _yargs = require("yargs");

var _yargs2 = _interopRequireDefault(_yargs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

var ES2015_TEMPLATE = "const Controller = require(\"maya\").Controller;\n\nmodule.exports = Controller.create({<% methods.forEach(function (methodName, idx, list) { %>\n\t<%= (idx !== 0 ? \"\\n\\t\" : \"\") %>\n\t<%= methodName %>(ctx) {\n\t\t\n\t}<%= idx !== (list.length - 1) ? \",\" : \"\" %><% }); %>\n});";

var ES2015_REST_TEMPLATE = "const RestController = require(\"maya\").RestController;\n\nmodule.exports = RestController.create({<% methods.forEach(function (methodName, idx, list) { %>\n\t_model : \"<%= modelName %>\",\n\t<%= (idx !== 0 ? \"\\n\\t\" : \"\") %>\n\t<%= methodName %>(ctx) {\n\t\t\n\t}<%= idx !== (list.length - 1) ? \",\" : \"\" %><% }); %>\n});";

/**
 * Parse `maya generate controller` args.
 * - `--indent (-i)` - indent type ("tab" or number of spaces)
 * - `--force (-f)` - Overwrite files even exist
 * - `--help (-h)` - Show help
 * @param {Array<String>} argv
 * @return {Object}
 */
function parseArgs(argv) {
    var parser = _yargs2.default.string("indent").alias("indent", "i").default("i", "2").describe("i", "Generate code indent type.(\"tab\" or number of spaces)").boolean("force").alias("force", "f").default("f", false).describe("f", "Overwrite files to even exist").boolean("rest").default("rest", false).describe("rest", "Create REST Controller. (Must specify with `--model` option.)").string("model").describe("model", "Specify assigned model for REST Controller.").boolean("help").alias("help", "h").strict();

    var options = parser.parse(argv);

    if (options.help) {
        console.log("Usage :\n" + "\tcontroller <controller_name> [<http_method>:]<method_name> [[<http_method>:]<method_name>...]");

        parser.showHelp("log");
        process.exit(0);
    }

    return options;
}

/**
 * @param {Array<String>} argv
 */
module.exports = function (argv) {
    var options = parseArgs(argv);

    var _options$_ = _toArray(options._);

    var controllerName = _options$_[0];

    var methodDefinitions = _options$_.slice(1);

    var controllerDir = _path2.default.join(process.cwd(), "server/controller/");
    var exportPath = _path2.default.join(controllerDir, controllerName + ".js");

    var indent = options.indent === "tab" ? "\t" : " ".repeat(parseInt(options.indent, 10));
    var template = options.rest ? _lodash2.default.template(ES2015_REST_TEMPLATE) : _lodash2.default.template(ES2015_TEMPLATE);

    var methods = methodDefinitions.map(function (methodDef) {
        var _methodDef$split = methodDef.split(":");

        var _methodDef$split2 = _slicedToArray(_methodDef$split, 2);

        var httpMethod = _methodDef$split2[0];
        var name = _methodDef$split2[1];

        if (!name) {
            var _ref = ["", httpMethod];
            httpMethod = _ref[0];
            name = _ref[1];
        }

        if (httpMethod !== "") {
            httpMethod = httpMethod + "_";
        }

        return httpMethod + name;
    });

    var content = template({
        methods: methods,
        modelName: options.model
    }).replace(/\t/gm, indent);

    // Check exists, if -f switch not taked.
    if (options.force === false && _fs2.default.existsSync(exportPath)) {
        var relatePath = exportPath.replace(process.cwd(), ".");

        console.error("\u001b[31m[maya.js] Controller '" + controllerName + "' already exists in '" + relatePath + "'");
        console.error("if you want overwrite it, please take '-f' switch.\u001b[m");
        process.exit(-1);
    }

    _fs2.default.writeFileSync(exportPath, content, "utf-8");
};