"use strict";

var _slicedToArray = (function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; })();

var _lodash = require("lodash");

var _lodash2 = _interopRequireDefault(_lodash);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _yargs = require("yargs");

var _yargs2 = _interopRequireDefault(_yargs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

var ES2015_TEMPLATE = "const Model = require(\"maya\").Model;\n\nmodule.exports = Model.create({<% methods.forEach(function (methodName, idx, list) { %>\n\t<%= (idx !== 0 ? \"\\n\\t\" : \"\") %>\n\t<%= methodName %>(req, res) {\n\t\t\n\t}<%= idx !== (list.length - 1) ? \",\" : \"\" %><% }); %>\n\nconst Model = require(\"maya\").Model;\n\nmodule.exports = Model.create({\n\n    // set true, if using schema type database.\n    schema : false,\n\n    // Specify table name if you want.\n    tableName : \"<%= tableName %>\",\n\n    // (optional) Using connection name\n    // if omitted, use \"default\" connection as default.\n    // if you want to use other (not \"default\") connection for this model,\n    // set connection name to `connection` property.\n    // (connection's defined in config/env/<environment>/database)\n    connection : \"default\",\n\n    // Field definition\n    attributes : {\n        // \"field_name\" : {\n        //     type        : String,\n        //     size        : Integer, // Optional\n        //     defaultsTo  : Any, // Optional\n        //     primaryKey  : Boolean, // Optional\n        //     autoIncrement : Boolean, // Optional\n        //     notNull     : Boolean, // Optional\n        //     unique      : Boolean, // Optional\n        //     index       : Boolean, // Optional\n        //     enum        : Array<String>, // Required when `type` is \"enum\"\n        //\n        //     // Relational field settion(Optional)\n        //     model : String, // Child model name (for one-one relation)\n        //     collection : String, // Child model name (for one-many, many-many relation)\n        //     via : String, // child field name for parent model primary key referecing.\n        // }\n        \"id\" : {\n            type : \"int\",\n            primaryKey : true,\n            autoIncrement : true,\n        },\n        \"displayId\" : {\n            type : \"string\",\n            size : 60,\n        },\n    },\n});\n\n});";

/**
 * Parse `maya generate controller` args.
 * - `--indent (-i)` - indent type ("tab" or number of spaces)
 * - `--force (-f)` - Overwrite files even exist
 * - `--help (-h)` - Show help
 * @param {Array<String>} argv
 * @return {Object}
 */
function parseArgs(argv) {
    var parser = _yargs2.default.string("indent").alias("indent", "i").default("i", "2").describe("i", "Generate code indent type.(\"tab\" or number of spaces)").boolean("force").alias("force", "f").default("f", false).describe("f", "Overwrite files to even exist").boolean("help").alias("help", "h").strict();

    var options = parser.parse(argv);

    if (options.help) {
        console.log("Usage :\n" + "\tmodel <model_name> <field>:<type> [<field>:<type>...]");

        parser.showHelp("log");
        process.exit(0);
    }

    return options;
}

/**
 * @param {Array<String>} argv
 */
module.exports = function (argv) {
    var options = parseArgs(argv);

    var _options$_ = _toArray(options._);

    var controllerName = _options$_[0];

    var methodDefinitions = _options$_.slice(1);

    var controllerDir = _path2.default.join(process.cwd(), "server/controller/");
    var exportPath = _path2.default.join(controllerDir, controllerName + ".js");

    var indent = options.indent === "tab" ? "\t" : " ".repeat(parseInt(options.indent, 10));
    var template = options.rest ? _lodash2.default.template(ES2015_REST_TEMPLATE) : _lodash2.default.template(ES2015_TEMPLATE);

    var methods = methodDefinitions.map(function (methodDef) {
        var _methodDef$split = methodDef.split(":");

        var _methodDef$split2 = _slicedToArray(_methodDef$split, 2);

        var httpMethod = _methodDef$split2[0];
        var name = _methodDef$split2[1];

        if (!name) {
            var _ref = ["", httpMethod];
            httpMethod = _ref[0];
            name = _ref[1];
        }

        if (httpMethod !== "") {
            httpMethod = httpMethod + "_";
        }

        return httpMethod + name;
    });

    var content = template({
        methods: methods,
        modelName: options.model
    }).replace(/\t/gm, indent);

    // Check exists, if -f switch not taked.
    if (options.force === false && _fs2.default.existsSync(exportPath)) {
        var relatePath = exportPath.replace(process.cwd(), ".");

        console.error("\u001b[31m[maya.js] Controller '" + controllerName + "' already exists in '" + relatePath + "'");
        console.error("if you want overwrite it, please take '-f' switch.\u001b[m");
        process.exit(-1);
    }

    _fs2.default.writeFileSync(exportPath, content, "utf-8");
};