"use strict";

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _lodash = require("lodash");

var _lodash2 = _interopRequireDefault(_lodash);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _glob = require("glob");

var _glob2 = _interopRequireDefault(_glob);

var _eventKit = require("event-kit");

var _deep = require("../utils/deep");

var deep = _interopRequireWildcard(_deep);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ConfigLoader = (function () {

    /**
     * @private
     * @property {Emitter} _emitter
     */

    /**
     * @private
     * @property {ModuleSwapper} _swapper
     */

    /**
     * @private
     * @property {Object} options
     * @property {String} options.configDir
     * @property {String} options.env
     */

    /**
     * @private
     * @property {Object} _configs
     */

    /**
     * @private
     * @property {Logger} logger
     */

    /**
     * @class ConfigLoader
     * @constructor
     * @param {ModuleSwapper} swapper
     * @param {Object} options
     * @param {String} options.configDir
     * @param {String} options.env
     */

    function ConfigLoader(swapper, options) {
        _classCallCheck(this, ConfigLoader);

        this._emitter = new _eventKit.Emitter();
        this._swapper = swapper;
        this.options = options;
        this.logger = options.logger;
        this._configs = Object.create(null);
    }

    /**
     * Load config files
     * @method load
     */

    _createClass(ConfigLoader, [{
        key: "load",
        value: function load() {
            var _this = this;

            var loadableExtensions = Object.keys(require.extensions).join(",");
            var allConfigFiles = _glob2.default.sync(this.options.configDir + "/**/*{" + loadableExtensions + "}");
            var envConfigFiles = allConfigFiles.filter(function (path) {
                return path.indexOf(_this.options.configDir + "env/" + _this.options.env + "/") === 0;
            });
            var commonConfigFiles = allConfigFiles.filter(function (path) {
                return envConfigFiles.indexOf(path) === -1;
            });

            var loadedConfigs = commonConfigFiles.map(function (filePath) {
                var namespace = _path2.default.parse(filePath).name;
                return _defineProperty({}, namespace, _this._swapper.require(filePath));
            }).concat(envConfigFiles.map(function (filePath) {
                var namespace = _path2.default.parse(filePath).name;
                return _defineProperty({}, namespace, _this._swapper.require(filePath));
            }));

            this._emitter.emit("reload");

            _lodash2.default.merge.apply(_lodash2.default, [this._configs].concat(_toConsumableArray(loadedConfigs)));
        }

        /**
         * Get config value
         * @param {String} key
         */

    }, {
        key: "get",
        value: function get(key, defaults) {
            return deep.get(this._configs, key, defaults);
        }

        /**
         * @method startWatch
         */

    }, {
        key: "startWatch",
        value: function startWatch() {
            var _this2 = this;

            _fs2.default.watch(this.options.configDir, { recursive: true }, function (ev, file) {
                _this2.logger.info("ConfigLoader", "Reloading config : %s", file);
                _this2.load();
            });
        }

        /**
         * @method observe
         * @param {String} key
         * @param {Function} listener
         */

    }, {
        key: "observe",
        value: function observe(key, listener) {
            var _this3 = this;

            var oldValue = this.get(key);
            this._emitter.on("reload", function () {
                var newValue = _this3.get(key);

                if (!_lodash2.default.isEqual(newValue, oldValue)) {
                    listener(newValue);
                }
            });
        }
    }]);

    return ConfigLoader;
})();

exports.default = ConfigLoader;