"use strict";

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _lodash = require("lodash");

var _lodash2 = _interopRequireDefault(_lodash);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _glob = require("glob");

var _glob2 = _interopRequireDefault(_glob);

var _pascalize = require("../utils/pascalize");

var _pascalize2 = _interopRequireDefault(_pascalize);

var _deep = require("../utils/deep");

var deep = _interopRequireWildcard(_deep);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ModelLoader = (function () {
    /**
     * @private
     * @property {Emitter} _emitter
     */

    /**
     * @private
     * @property {ModuleSwapper} _swapper
     */

    /**
     * @private
     * @property {Object} options
     * @property {String} options.modelDir
     */

    /**
     * @private
     * @property {Object} _configs
     */

    /**
     * @private
     * @property {Logger} logger
     */

    /**
     * @class ConfigLoader
     * @constructor
     * @param {ModuleSwapper} swapper
     * @param {Object} options
     * @param {String} options.modelDir
     * @param {String} options.modelLogicsDir
     */

    function ModelLoader(swapper, options) {
        _classCallCheck(this, ModelLoader);

        this._swapper = swapper;
        this.options = options;
        this.logger = options.logger;
        this._modelInfos = {};
    }

    /**
     * Load model files
     * @method load
     */

    _createClass(ModelLoader, [{
        key: "load",
        value: function load() {
            var _this = this;

            var loadableExtensions = Object.keys(require.extensions).join(",");

            // Load models
            var modelSchemaFiles = _glob2.default.sync(this.options.modelDir + "/**/*{" + loadableExtensions + "}");

            modelSchemaFiles.forEach(function (fullPath) {
                var pathInfo = _path2.default.parse(fullPath.slice(_this.options.modelDir.length));
                var identity = pathInfo.dir + (pathInfo.dir !== "" ? "/" : "") + pathInfo.name;
                var model = _this._swapper.require(fullPath);
                var modelId = (pathInfo.dir !== "" ? pathInfo.dir + "/" : "") + pathInfo.name;;

                _this._modelInfos[modelId] = { identity: identity, model: fullPath };
            });

            // Load logics and assign to model.
            var logicFiles = _glob2.default.sync(this.options.modelLogicsDir + "/**/*{" + loadableExtensions + "}");

            logicFiles.forEach(function (fullPath) {
                var relativePath = fullPath.slice(_this.options.modelLogicsDir.length);
                var pathInfo = _path2.default.parse(relativePath);
                var modelId = (pathInfo.dir !== "" ? pathInfo.dir + "/" : "") + pathInfo.name;

                if (!_this._modelInfos[modelId]) {
                    _this.logger.warn("ModelLoader", "There is no model that corresponds to the Logic.(for app/logics/model-logic/" + relativePath + ")");
                    return;
                }

                _this._modelInfos[modelId].logic = fullPath;
            });
        }

        /**
         * Set models into Waterline
         * @param {Waterline} waterline
         * @param {Object} waterline connection options
         * @return {Promise<Object>} waterline connected Models
         */

    }, {
        key: "setupModels",
        value: (function () {
            var ref = _asyncToGenerator(function* (waterline, options) {
                var _this2 = this;

                _lodash2.default.each(this._modelInfos, function (info) {
                    var modelPath = info.model;
                    var logicPath = info.logic;

                    var model = _this2._swapper.require(modelPath);
                    var logic = logicPath ? _lodash2.default.cloneDeep(_this2._swapper.require(logicPath)) : {};
                    _this2._wrapLogicMethods(logic);

                    waterline.loadCollection(model.toWaterlineCollection(logic, {
                        identity: info.identity
                    }));
                });

                return new Promise(function (resolve, reject) {
                    waterline.initialize(options, function (err, result) {
                        if (err) {
                            reject(err);
                            return;
                        }

                        var dsPattern = new RegExp("\\" + _path2.default.sep, "g");
                        var models = Object.create(null);
                        _lodash2.default.each(result.collections, function (model, identity) {
                            var dirSeparated = identity.replace(dsPattern, ".").split(".");
                            var pascalizeKey = dirSeparated.map(function (v) {
                                return (0, _pascalize2.default)(v);
                            }).join(".");

                            deep.set(models, pascalizeKey, model);
                        });

                        resolve(models);
                    });
                });
            });

            return function setupModels(_x, _x2) {
                return ref.apply(this, arguments);
            };
        })()
    }, {
        key: "_wrapLogicMethods",
        value: function _wrapLogicMethods(obj) {
            _lodash2.default.each(obj, function (prop, name) {
                if (!_lodash2.default.isFunction(prop)) return;

                obj[name] = function () {
                    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
                        args[_key] = arguments[_key];
                    }

                    return prop.apply(undefined, [this].concat(args));
                };
            });

            return obj;
        }

        /**
         * @method startWatch
         */

    }, {
        key: "startWatch",
        value: function startWatch() {
            var _this3 = this;

            _fs2.default.watch(this.options.modelDir, { recursive: true }, function (ev, file) {
                _this3.logger.info("ModelLoader", "Reloading config : %s", file);
                _this3.load();
            });
        }
    }]);

    return ModelLoader;
})();

exports.default = ModelLoader;