"use strict";

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _lodash = require("lodash");

var _lodash2 = _interopRequireDefault(_lodash);

var _eventemitter = require("eventemitter3");

var _eventemitter2 = _interopRequireDefault(_eventemitter);

var _util = require("util");

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var logLevel = {
    "silly": {
        level: 0,
        logger: console.log.bind(console),
        label: "\u001b[37m[%s] \u001b[m",
        color: "\u001b[37m"
    },
    "verbose": {
        level: 1,
        logger: console.log.bind(console),
        label: "[%s] ",
        color: ""
    },
    "info": {
        level: 3,
        logger: console.info.bind(console),
        label: "\u001b[36;1m[%s] \u001b[m",
        color: "\u001b[36m"
    },
    "debug": {
        level: 2,
        logger: console.info.bind(console),
        label: "\u001b[32;1m[%s] \u001b[m",
        color: "\u001b[32m"
    },
    "warn": {
        level: 4,
        logger: console.error.bind(console),
        label: "\u001b[33;1m[%s] \u001b[m",
        color: "\u001b[33m"
    },
    "error": {
        level: 5,
        logger: console.error.bind(console),
        label: "\u001b[31;1m[%s] \u001b[m",
        color: "\u001b[31m"
    }
};

var Logger = (function (_Emitter) {
    _inherits(Logger, _Emitter);

    _createClass(Logger, null, [{
        key: "logLevel",
        get: function get() {
            return logLevel;
        }

        /**
         * @class Logger
         * @constructor
         * @param {Object} options
         * @param {Number} options.logLevel logging level (log specified level or more)
         * @param {Boolean} options.paused Pause log printing.
         */

    }]);

    function Logger(options) {
        _classCallCheck(this, Logger);

        var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Logger).call(this));

        _this.options = _lodash2.default.defaults({}, options, {
            logLevel: logLevel.info.level,
            paused: false
        });

        _this._buffer = [];

        Object.keys(logLevel).forEach(function (logType) {
            var logger = logLevel[logType].logger;
            var labeler = logLevel[logType].label;
            var color = logLevel[logType].color;
            var level = logLevel[logType].level;

            _this[logType] = function (label, message) {
                for (var _len = arguments.length, more = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
                    more[_key - 2] = arguments[_key];
                }

                if (level < _this.options.logLevel) {
                    return;
                }

                var logLabel = _util2.default.format(labeler, label);
                var plainMessage = _util2.default.format.apply(_util2.default, [message].concat(more));
                var logMessage = color + plainMessage + "\u001b[m";

                if (_this.options.paused) {
                    _this._buffer.push({ logger: logger, level: level, message: logLabel + logMessage });
                } else {
                    logger(logLabel + logMessage);
                }

                _this.emit("log", {
                    type: logType,
                    level: level,
                    label: label,
                    message: plainMessage
                });
            };
        });
        return _this;
    }

    _createClass(Logger, [{
        key: "setLogLevel",
        value: function setLogLevel(level) {
            if (typeof level === "string") {
                this.options.logLevel = logLevel[level].level;
            } else if (typeof level === "number") {
                this.options.logLevel = level;
            }
        }

        /**
         * Pause error printing (not "log" event emitting)
         */

    }, {
        key: "pause",
        value: function pause() {
            this.options.paused = true;
        }

        /**
         * Resume error printing
         */

    }, {
        key: "resume",
        value: function resume() {
            var _this2 = this;

            this.options.paused = false;
            this._buffer.forEach(function (logItem) {
                if (logItem.level < _this2.options.logLevel) {
                    return;
                }
                logItem.logger(logItem.message);
            });
        }
    }]);

    return Logger;
})(_eventemitter2.default);

exports.default = Logger;