"use strict";

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _lodash = require("lodash");

var _lodash2 = _interopRequireDefault(_lodash);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _waterline = require("waterline");

var _waterline2 = _interopRequireDefault(_waterline);

var _yargs = require("yargs");

var _yargs2 = _interopRequireDefault(_yargs);

var _co = require("co");

var _co2 = _interopRequireDefault(_co);

var _logger = require("./logger");

var _logger2 = _interopRequireDefault(_logger);

var _fileWatcher = require("./file-watcher");

var _fileWatcher2 = _interopRequireDefault(_fileWatcher);

var _moduleSwapper = require("./module-swapper");

var _moduleSwapper2 = _interopRequireDefault(_moduleSwapper);

var _configLoader = require("./loader/config-loader");

var _configLoader2 = _interopRequireDefault(_configLoader);

var _modelLoader = require("./loader/model-loader");

var _modelLoader2 = _interopRequireDefault(_modelLoader);

var _server2 = require("./server");

var _server3 = _interopRequireDefault(_server2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var VERSION = require(_path2.default.join(__dirname, "../package.json")).version;

/**
 * Initialize application backends.(Load config, Build assets, Start server)
 * @class Maya
 */

var Maya = (function () {
    _createClass(Maya, null, [{
        key: "parseArgs",

        /**
         * @param {String} argv argv, pre excuted `process.argv.slice(2)`
         * @return {Object}
         */
        value: function parseArgs(argv) {
            var parser = (0, _yargs2.default)().boolean("interactive").alias("interactive", "i").describe("i", "Run with REPL").string("port").describe("port", "Listening port number. (It's overrides configured port number)").boolean("no-watch").describe("no-watch", "No watching Model, View, Controller changing.").string("env").default("env", "devel").describe("env", "Specify running environment (production, devel, test).").help("help").alias("help", "h").strict();

            var args = parser.parse(argv);

            var watch = !args.w;
            if (args.w == null && args.env === Maya.ENV_DEVEL) {
                watch = true;
            }

            return {
                interactive: args.interactive,
                port: args.port != null ? parseInt(args.port, 10) : null,
                watch: watch,
                env: args.env
            };
        }

        /**
         * @private
         * @property {Object} _options
         */

        /**
         * @property {Logger} logger
         */

        /**
         * @property {FileWatcher} watcher
         */

        /**
         * @property {ConfigLoader} config
         */

        /**
         * @property {ModelLoader} _modelLoader
         */

        /**
         * @property {ModuleSwapper} swapper
         */

        /**
         * @property {Waterline} waterline
         */

        /**
         * @property {String} keys
         */

        /**
         * @param {Object} options
         * @param {String} options.env Application running environment
         * @param {String} options.appRoot Application running path
         * @param {Number} options.port Listening port number
         * @param {Object} options.routes route definition object
         * @param {Boolean} options.watch if true, enable watch for Controller, Model, View
         */

    }, {
        key: "VERSION",
        get: function get() {
            return VERSION;
        }
    }, {
        key: "ENV_DEVEL",
        get: function get() {
            return "devel";
        }
    }, {
        key: "ENV_PRODUCTION",
        get: function get() {
            return "production";
        }
    }]);

    function Maya(options) {
        var _this = this;

        _classCallCheck(this, Maya);

        this._options = options || {};
        // this._options = _.defaults(options, {
        //     env     : "devel",
        //     appRoot : process.cwd() + path.sep,
        //     watch   : true,
        //     port    : null
        // });

        // application logger
        this.logger = new _logger2.default({
            logLevel: 0,
            paused: true
        });

        // file watcher
        this.watcher = new _fileWatcher2.default();

        // SwappableModule loader
        this.swapper = new _moduleSwapper2.default({
            logger: this.logger,
            watch: this._options.watch
        });

        // config accessor
        this.config = new _configLoader2.default(this.swapper, {
            logger: this.logger,
            configDir: _path2.default.join(this._options.appRoot, "config/"),
            env: this._options.env
        });

        // Load configs
        this.config.load({ watch: this._options.watch });

        // Model loader
        this._modelLoader = new _modelLoader2.default(this.swapper, {
            logger: this.logger,
            modelDir: _path2.default.join(this._options.appRoot, "models/"),
            modelLogicsDir: _path2.default.join(this._options.appRoot, "logics/model-logic/")
        });

        // database connector
        this.waterline = new _waterline2.default();

        // express instance handler
        this.server = new _server3.default({
            swapper: this.swapper,
            logger: this.logger
        });

        Object.defineProperty(this, "keys", {
            set: function set(value) {
                return _this.server._koa.keys = value;
            },
            get: function get() {
                return _this.server._koa.keys;
            }
        });

        global.maya = this;
    }

    /**
     * Start application
     * @method start
     */

    _createClass(Maya, [{
        key: "start",
        value: (function () {
            var ref = _asyncToGenerator(function* () {
                try {
                    // set log level
                    this.logger.setLogLevel(this.config.get("maya.log.level"));
                    this.logger.resume();

                    // Load model definitions
                    this._modelLoader.load({ watch: this._options.watch });

                    // Link models to waterline and expose models
                    this.models = yield this._modelLoader.setupModels(this.waterline, this.config.get("database"));

                    // Run build script (`app/build.js`)
                    this.logger.info("App#start", "Waiting for build...");
                    yield this._buildScripts();
                    this.logger.info("App#start", "End build");

                    // get socket.io host object
                    this.sockets = this.server.getSocketIo();

                    // if watch option enabled, start chnages watching.
                    if (this._options.watch) {
                        this.config.startWatch();
                        this._startAssetsWatching();
                    }

                    // Decide listening port
                    var listeningPort = this._options.port == null ? this.config.get("maya.server.port") : this._options.port | 0;

                    // Start listening
                    yield this.server.start({
                        config: this.config,
                        appRoot: this._options.appRoot,
                        port: listeningPort,
                        routes: this.config.get("routes"),
                        watch: this._options.watch
                    });

                    this.logger.info("Server", "<maya.js start on port " + listeningPort + " in " + this._options.env + " environment.>");
                } catch (e) {
                    this.logger.error("App#start", e.message + "\n" + e.stack);
                    process.exit(-1);
                }
            });

            return function start() {
                return ref.apply(this, arguments);
            };
        })()

        /**
         * attach middleware to express
         * @method use
         */

    }, {
        key: "use",
        value: function use() {
            var _server;

            (_server = this.server).use.apply(_server, arguments);
        }
    }, {
        key: "_buildScripts",
        value: (function () {
            var ref = _asyncToGenerator(function* () {
                this.logger.info("App Builder", "Run build.js");

                var buildScript = _path2.default.join(this._options.appRoot, "build.js");
                var builder = this.swapper.require(buildScript, require);
                var returns = builder(this._options.env);

                yield (0, _co2.default)(returns);
            });

            return function _buildScripts() {
                return ref.apply(this, arguments);
            };
        })()
    }, {
        key: "_startAssetsWatching",
        value: function _startAssetsWatching() {
            var _this2 = this;

            var staticDir = _path2.default.join(this._options.appRoot, ".tmp/");
            var stylesDir = _path2.default.join(staticDir, "styles/");
            var controllersDir = _path2.default.join(this._options.appRoot, "controller/");
            var viewsDir = _path2.default.join(this._options.appRoot, "views/");

            if (!_fs2.default.existsSync(staticDir)) {
                _fs2.default.mkdirSync(staticDir);
            }

            // Request swapping links
            var swap = _lodash2.default.debounce(function (type, fileUrl) {
                _this2.sockets.to("__maya__").emit("__maya__.swap", { fileType: "css", fileUrl: fileUrl });
            }, 500, { maxWait: 5000 });

            // Request page reloading
            var reload = _lodash2.default.debounce(function (file) {
                _this2.sockets.to("__maya__").emit("__maya__.reload");
            }, 500, { maxWait: 5000 });

            // watch static assets
            this.watcher.watch(staticDir, function (event, file) {
                if (/^styles\/.+/.test(file)) {
                    // if css file updated
                    swap("css", "/" + file);
                } else {
                    reload();
                }
            });

            // watch controllers changes
            this.watcher.watch(controllersDir, reload);

            // watch view changes
            this.watcher.watch(viewsDir, reload);
        }
    }]);

    return Maya;
})();

exports.default = Maya;