"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = errorHandler;

var _prettyLog = require("../utils/pretty-log");

var prettyLog = _interopRequireWildcard(_prettyLog);

var _jade = require("jade");

var _jade2 = _interopRequireDefault(_jade);

var _notAllowed = require("../exception/not-allowed");

var _notAllowed2 = _interopRequireDefault(_notAllowed);

var _notfound = require("../exception/notfound");

var _notfound2 = _interopRequireDefault(_notfound);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

var VIEW_GENERAL = __dirname + "/../views/general-error.jade";
var VIEW_500 = __dirname + "/../views/500.jade";

function errorHandler() {
    return function* (next) {
        var err;

        try {
            yield next;
        } catch (e) {
            err = e;
        }

        if (this.status === 403) {
            this.type = "text/html; charset=UTF-8";
            this.body = _jade2.default.compileFile(VIEW_GENERAL)({
                title: "403 Forbidden",
                message: "Forbidden"
            });
        }

        if (this.status === 404) {
            this.type = "text/html; charset=UTF-8";
            this.body = _jade2.default.compileFile(VIEW_GENERAL)({
                title: "404 Not Found",
                message: "Your requested resource moved or removed."
            });
        }

        if (err) {
            this.type = "text/html; charset=UTF-8";
            this.status = 500;

            // prettyLog.error("Internal server error", err);

            var lines = err.stack.split("\n");
            var title = lines.shift();
            this.body = _jade2.default.compileFile(VIEW_500)({
                title: "500 // Internal Server Error",
                error: {
                    title: title,
                    stack: lines.map(function (item) {
                        return item.replace(/^[\s\t]+/g, "");
                    })
                }
            });
        }

        return yield next;
    };
};