"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = RestController;

var _lodash = require("lodash");

var _lodash2 = _interopRequireDefault(_lodash);

var _extend = require("./utils/extend");

var _extend2 = _interopRequireDefault(_extend);

var _deep = require("./utils/deep");

var deep = _interopRequireWildcard(_deep);

var _controller = require("./controller");

var _controller2 = _interopRequireDefault(_controller);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Write class uses function
// For listup RestController methods in Router.
function RestController() {
    if (_lodash2.default.isFunction(this._init)) {
        this._init();
    }
}

/**
 * @static
 * @method create
 * @param {Object} proto
 * @param {String|Model} proto._model
 * @return {RestController}
 */
RestController.create = function (proto) {
    var SubClass = (0, _extend2.default)(proto, RestController);
    var instance = new SubClass();

    if (_lodash2.default.isFunction(instance._init)) {
        instance._init();
    }

    if (typeof proto._model !== "string") {
        throw new Error("Model property must be (dot-notation) model name string for reference `maya.models`.");
    }

    return instance;
};

RestController.prototype = _lodash2.default.extend(Object.create(_controller2.default), {
    _init: function _init() {
        this.model = deep.get(maya.models, this._model);
    },
    _dispose: function _dispose() {
        delete this.model;
    },

    // Delegation handler for Router register

    "get_index": function get_index(ctx) {
        return this._list(ctx);
    },
    "get_:id": function get_Id(ctx) {
        return this._get(ctx);
    },
    "post_index": function post_index(ctx) {
        return this._post(ctx);
    },
    "delete_:id": function delete_Id(ctx) {
        return this._delete(ctx);
    },
    "put_:id": function put_Id(ctx) {
        return this._put(ctx);
    },
    "patch_:id": function patch_Id(ctx) {
        return this._put(ctx);
    },

    // Real handlers

    _list: function* _list(ctx) {
        ctx.body = yield this.model.find();
    },
    _get: function* _get(ctx) {
        ctx.body = yield this.model.find(req.params.id)[0];
    },
    _post: function* _post(ctx) {
        ctx.body = yield this.model.create(req.body)[0];
    },
    _delete: function* _delete(ctx) {
        ctx.body = yield this.model.destroy(req.params.id)[0];
    },
    _put: function* _put(ctx) {
        ctx.body = yield this.model.update(req.params.id, req.body)[0];
    }
});