"use strict";

var _slicedToArray = (function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; })();

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _lodash = require("lodash");

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// TODO: Replace searching algorithm to
//  fragments->method pattern. (Now is `method->fragments` implement).
// For simplify and reduce searching times.

var RouteTree = (function () {
    /**
     * @class RouteTree
     * @constructor
     * @param {Object?} routeTree initial route tree
     */

    function RouteTree(routeTree) {
        _classCallCheck(this, RouteTree);

        this.clear();
        routeTree && (this.tree = routeTree);
    }

    /**
     * Clear holding route tree
     * @method clear
     */

    _createClass(RouteTree, [{
        key: "clear",
        value: function clear() {
            this.tree = Object.create(null);
        }

        /**
         * @method mergeFlattenTree
         * @param {Array<Array<string, Array<string>, Object>} routeList
         *  [0] -> httpMethod
         *  [1] -> url fragments pattern
         *  [2] -> Controller information
         */

    }, {
        key: "mergeFlattenTree",
        value: function mergeFlattenTree(routeList) {
            var routeTree = Object.create(null);

            _lodash2.default.each(routeList, function (_ref, idx, list) {
                var _ref2 = _slicedToArray(_ref, 3);

                var httpMethod = _ref2[0];
                var urlFragments = _ref2[1];
                var handlerInfo = _ref2[2];

                var targetNode;
                var validUrlFragments = urlFragments.filter(function (f) {
                    return f !== "/" && f !== "" & !!f;
                });

                // Build route tree
                var pattern = [];
                validUrlFragments.reduce(function (parentNode, fragment) {
                    pattern.push(fragment);
                    parentNode[fragment] = parentNode[fragment] || Object.create(null);

                    targetNode = parentNode[fragment];
                    targetNode._isParam = fragment[1] === ":";
                    targetNode._pattern = pattern.join("");

                    return targetNode;
                }, routeTree);

                targetNode._handlers = targetNode._handlers || Object.create(null);
                targetNode._handlers[httpMethod] = handlerInfo;
            });

            this.merge(routeTree);
        }

        /**
         * @method merge
         * @param {Array<Object>}
         */

    }, {
        key: "merge",
        value: function merge() {
            for (var _len = arguments.length, trees = Array(_len), _key = 0; _key < _len; _key++) {
                trees[_key] = arguments[_key];
            }

            // TODO: Implement merge with conflict checking
            return _lodash2.default.merge.apply(_lodash2.default, [this.tree].concat(trees));
        }

        /**
         * @method findMatchController
         * @param {String} httpMethod
         * @param {String} url likes "/path/to/route"
         * @return {Object}
         *  controller : Path to controller file
         *  method : Calling method name
         *  url : matched url pattern
         */

    }, {
        key: "findMatchController",
        value: function findMatchController(httpMethod, url) {
            var _this = this;

            var routeTree = arguments.length <= 2 || arguments[2] === undefined ? this.tree : arguments[2];

            httpMethod = httpMethod.toLowerCase();

            var urlFragments = url.replace(/(^\/|\/$)/g, "").split("/").filter(function (fragment) {
                return fragment !== "";
            }).map(function (fragment) {
                return "/" + fragment;
            });

            var candidateNodes = urlFragments.reduce(function (candidateNodes, currentFragment) {
                return _this._findMatchNodes(candidateNodes, currentFragment);
            }, [this.tree]);

            var matchedNode = candidateNodes[0];
            var candidateHandlers = matchedNode != null ? matchedNode._handlers : null;

            // Find `/index` handler when handler not find in matchedNode.
            var handlers = candidateHandlers;
            if (handlers == null && matchedNode && matchedNode["/index"]) {
                handlers = matchedNode["/index"]._handlers;
            }

            if (candidateNodes.length > 1) {
                console.warn("Can't resolve route.", candidateNodes.map(function (node) {
                    return node._handlers;
                }));
                return;
            }

            if (matchedNode == null || handlers == null) return;
            if (handlers[httpMethod] == null && handlers.all == null) return;

            var matchedNodeInfo = handlers[httpMethod] || handlers.all;

            return {
                controller: matchedNodeInfo.controller,
                method: matchedNodeInfo.method,
                pattern: matchedNode._pattern
            };
        }

        /**
         * @private
         * @param {Array<Object>} nodes
         * @param {String} fragment a URL fragment likes "/fragment"
         */

    }, {
        key: "_findMatchNodes",
        value: function _findMatchNodes(nodes, fragment) {
            var firstMatches = _lodash2.default.reduce(nodes, function (matches, node) {
                _lodash2.default.each(node, function (child, key) {
                    if (key === fragment) {
                        matches.statics.push(child);
                    } else if (child._isParam) {
                        matches.dynamics.push(child);
                    }
                });

                return matches;
            }, { dynamics: [], statics: [] });

            if (firstMatches.statics.length === 1) {
                return firstMatches.statics;
            } else if (firstMatches.dynamics.length === 1) {
                return firstMatches.dynamics;
            }

            return [].concat(_toConsumableArray(firstMatches.statics), _toConsumableArray(firstMatches.dynamics));
        }
    }]);

    return RouteTree;
})();

exports.default = RouteTree;