"use strict";

var _slicedToArray = (function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; })();

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _co = require("co");

var _co2 = _interopRequireDefault(_co);

var _lodash = require("lodash");

var _lodash2 = _interopRequireDefault(_lodash);

var _glob = require("glob");

var _glob2 = _interopRequireDefault(_glob);

var _metacarattere = require("metacarattere");

var _metacarattere2 = _interopRequireDefault(_metacarattere);

var _router = require("router");

var _router2 = _interopRequireDefault(_router);

var _prettyLog = require("./utils/pretty-log");

var PrettyLog = _interopRequireWildcard(_prettyLog);

var _controller = require("./controller");

var _controller2 = _interopRequireDefault(_controller);

var _restController = require("./rest-controller");

var _restController2 = _interopRequireDefault(_restController);

var _moduleSwapper = require("./module-swapper");

var _moduleSwapper2 = _interopRequireDefault(_moduleSwapper);

var _routeTree = require("./route-tree");

var _routeTree2 = _interopRequireDefault(_routeTree);

var _notfound = require("./exception/notfound");

var _notfound2 = _interopRequireDefault(_notfound);

var _serverError = require("./exception/server-error");

var _serverError2 = _interopRequireDefault(_serverError);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } step("next"); }); }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Router = (function () {

    /**
     * @property {ModuleSwapper} swapper
     */
    // swapper;

    /**
     * @property {Object} routes
     *  Structure likes:
     *    {
     *      "http method" : {
     *        "/url_fragment" : {
     *          isParam : false,
     *          controller : {
     *              controller : "path to controller",
     *              method : "handler method",
     *          }
     *          "/url_fragment" : {...}
     *        },
     *        "/:param_name" : {
     *          isParam : true,
     *          "/url_fragment" : {...}
     *          "/:param_name" : {
     *            "/:param_name" : {
     *              controller : { ... }
     *            }
     *          }
     *        }
     *      }
     *    }
     */
    // routes;

    /**
     * @property {Object} options
     * @property {Boolean} options.watch
     * @property {Object} options.routes
     * @property {String} options.controllerDir
     */

    /**
     * @property {routeTree} routeTree
     */

    /**
     * @class router
     * @constructor
     * @param {ModuleSwapper} swapper
     * @param {Object} options
     * @param {Logger} options.logger
     */

    function Router(swapper, options) {
        _classCallCheck(this, Router);

        this.swapper = swapper;
        this.options = options;
        this.routeTree = new _routeTree2.default();
    }

    /**
     * @param {Object} options
     * @param {Object} options.routes
     * @param {String} options.controllerDir
     */

    _createClass(Router, [{
        key: "load",
        value: function load(options) {
            this.routeTree.mergeFlattenTree(this._prefetchControllers(options.controllerDir));
            this.routeTree.mergeFlattenTree(this._prefetchRoutes(options));
        }

        /**
         * @param {String} controllerDir with "/" prefix
         * @return {Map}
         */

    }, {
        key: "_prefetchControllers",
        value: function _prefetchControllers(controllerDir) {
            var _this = this;

            var loadableExtensions = Object.keys(require.extensions).map(function (dotext) {
                return dotext.slice(1);
            }).join(",");
            var controllerPaths = _glob2.default.sync(controllerDir + "/**/*.{" + loadableExtensions + "}");

            return controllerPaths.map(function (fullPath) {
                // get controller relative path from `app/controller/`
                return [fullPath, fullPath.slice(controllerDir.length)];
            }).map(function (_ref) {
                var _ref2 = _slicedToArray(_ref, 2);

                var fullPath = _ref2[0];
                var relativePath = _ref2[1];

                // Survey controller's implemented handlers.
                // And returns [fullPath, relatePath, methods].
                // `relativePath` is relate path from `app/controller/`

                var controller = _this.swapper.require(fullPath, require);
                _this._isValidController(controller);

                var validMethods = _this._lookupValidHandler(controller);

                return [fullPath, relativePath, validMethods];
            }).reduce(function (urlMaps, _ref3) {
                var _ref4 = _slicedToArray(_ref3, 3);

                var fullControllerPath = _ref4[0];
                var relativePath = _ref4[1];
                var methodNames = _ref4[2];

                // iterate per Controller.
                // Build url <> controller#method map looks
                // ["httpMethod", ["/url", "/fragment"], {handler info}]

                var pathInfo = _path2.default.parse(relativePath);
                var controllerName = pathInfo.name;
                var urlFragments = pathInfo.dir.split("/").filter(function (fragment) {
                    return fragment !== "";
                }).map(function (fragment) {
                    return "/" + fragment;
                });

                if (controllerName === "_root_" && urlFragments.length !== 0) {
                    throw new Error("_root_ Controller only deployment as `app/controller/_root_.js`");
                }

                if (controllerName !== "_root_") {
                    // "app/controller/_root_.js" mapped to `example.com/*`
                    // otherwise mapped to "exapmle.com/${controllerName}/*"
                    urlFragments.push("/" + controllerName);
                }

                methodNames.forEach(function (methodName) {
                    var _methodName$split = methodName.split("_");

                    var _methodName$split2 = _toArray(_methodName$split);

                    var httpMethod = _methodName$split2[0];

                    var splitedMethodName = _methodName$split2.slice(1);

                    var urlName = splitedMethodName.join("_");
                    var mappedUrl = urlFragments.slice(0);

                    if (splitedMethodName.length === 0) {
                        // Case of method name likes "someMethodName"
                        urlName = httpMethod;
                        httpMethod = "all";
                    }

                    if (controllerName === "_root_") {
                        mappedUrl[0] = "/" + urlName;
                    } else {
                        mappedUrl.push("/" + urlName);
                    }

                    urlMaps.push([httpMethod, mappedUrl, {
                        controller: fullControllerPath,
                        method: methodName
                    }]);
                });

                return urlMaps;
            }, []);
        }

        /**
         * @private
         * @method _lookupValidHandler
         * @param {Controller} controller method lookup target
         * @return {Array<string>} lookuped valid method names
         */

    }, {
        key: "_lookupValidHandler",
        value: function _lookupValidHandler(controller) {
            // Lookup only first level prototype's public methods.

            // It's for security reason.
            // if accept access to instance method / property or deep prototype methods.
            // it could allow calls some methods to themalicious attacker.
            // if callled method is destructive (likes Controller._dispose) or property.
            // There is a possibility that the cause to services stopping or mismatch datas.
            var proto = Object.getPrototypeOf(controller);
            var protoMethodAndProps = Object.keys(proto);

            var validMethods = protoMethodAndProps.filter(function (methodName) {
                // ignore private methods
                if (methodName[0] === "_") {
                    return false;
                }

                // ignore properties
                return _lodash2.default.isFunction(proto[methodName]);
            });

            if (controller instanceof _restController2.default === false) {
                return validMethods;
            }

            // if conntroller is a RestController
            // List up more deep methods
            var restProto = Object.getPrototypeOf(proto);
            var restMethods = Object.keys(restProto);

            validMethods.push.apply(validMethods, _toConsumableArray(restMethods.filter(function (methodName) {
                // ignore private methods
                if (methodName[0] === "_") {
                    return false;
                }

                // ignore properties
                return _lodash2.default.isFunction(restProto[methodName]);
            })));

            return validMethods;
        }
    }, {
        key: "_prefetchRoutes",
        value: function _prefetchRoutes(options) {
            var _this2 = this;

            var routes = options.routes;
            var controllerDir = options.controllerDir;

            return _lodash2.default.map(routes, function (handler, routeConf) {
                // Build url <> controller#method map looks
                // ["httpMethod", ["/url", "/fragment"], {handler info}]

                // handler expects string looks "controller.method" or "path/to/controller.method"

                var _routeConf$split = routeConf.split(" ");

                var _routeConf$split2 = _slicedToArray(_routeConf$split, 2);

                var httpMethod = _routeConf$split2[0];
                var url = _routeConf$split2[1];

                var _handler$split = handler.split(".");

                var _handler$split2 = _slicedToArray(_handler$split, 2);

                var controllerName = _handler$split2[0];
                var methodName = _handler$split2[1];

                var unresolvedControllerPath = _path2.default.join(controllerDir, controllerName);
                var fullControllerPath;

                if (url === undefined) {
                    url = httpMethod;
                    httpMethod = "all";
                }

                //
                // Lookup Controller
                //
                try {
                    fullControllerPath = require.resolve(unresolvedControllerPath);
                } catch (e) {
                    throw new Error("Routed controller doesn't exists. (url: " + url + ", name: " + controllerName + ")");
                }

                // Check controller validness.
                var controller = _this2.swapper.require(fullControllerPath);
                _this2._isValidController(controller);

                var urlFragments = url.split("/").map(function (fragment) {
                    return "/" + fragment;
                }).filter(function (fragment) {
                    return fragment !== "/";
                });

                return [httpMethod, urlFragments, {
                    controller: fullControllerPath,
                    method: methodName
                }];
            });
        }

        /**
         * @param {http.IncomingMessage} req
         * @param {http.ServerResponse} res
         */

    }, {
        key: "handle",
        value: (function () {
            var ref = _asyncToGenerator(function* (ctx) {
                var method = ctx.method;
                var url = ctx.url;

                try {
                    var target = this.routeTree.findMatchController(method, url);

                    if (!target) {
                        ctx.status = 404;
                        return;
                    }

                    // Assgin URL palaceholder parameters
                    var matcher = new _metacarattere2.default(target.pattern);
                    var params = matcher.parse(url);
                    ctx.params = ctx.params || {};
                    _lodash2.default.assign(ctx.params, params, {});

                    // launch
                    yield this._launchController(target, ctx);
                } catch (e) {
                    // All Exceptions Handled By `server-error` middleware.
                    throw e;
                }
            });

            return function handle(_x) {
                return ref.apply(this, arguments);
            };
        })()
    }, {
        key: "_isValidController",
        value: function _isValidController(controller) {
            // Check controller extending correctness
            var proto = Object.getPrototypeOf(controller);

            if (!proto instanceof _controller2.default) {
                throw new Error("Controller must be create via Controller.create. (for " + fullControllerPath + ")");
            }
        }
        /**
         * @param {Object} target
         * @param {String} target.controller
         * @param {String} target.method
         * @param {String} target.urlPattern
         * @param {String} target.httpMethod LowerCased request http method
         * @param {http.IncomingMessage} req
         * @param {http.ServerResponse} res
         */

    }, {
        key: "_launchController",
        value: (function () {
            var ref = _asyncToGenerator(function* (target, ctx) {
                var controller = this.swapper.require(target.controller, require);
                var proto = Object.getPrototypeOf(controller);

                if (_lodash2.default.isFunction(proto[target.method])) {
                    if (_lodash2.default.isFunction(proto._before)) {
                        yield (0, _co2.default)(proto._before.call(controller, ctx));
                    }

                    yield (0, _co2.default)(proto[target.method].call(controller, ctx));

                    if (_lodash2.default.isFunction(proto._after)) {
                        yield (0, _co2.default)(proto._after.call(controller, ctx));
                    }

                    return;
                }

                throw new _notfound2.default();
            });

            return function _launchController(_x2, _x3) {
                return ref.apply(this, arguments);
            };
        })()
    }]);

    return Router;
})();

exports.default = Router;