"use strict";

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

// Middleware

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _http = require("http");

var _http2 = _interopRequireDefault(_http);

var _https = require("https");

var _https2 = _interopRequireDefault(_https);

var _lodash = require("lodash");

var _lodash2 = _interopRequireDefault(_lodash);

var _koa2 = require("koa");

var _koa3 = _interopRequireDefault(_koa2);

var _socket = require("socket.io");

var _socket2 = _interopRequireDefault(_socket);

var _router = require("./router");

var _router2 = _interopRequireDefault(_router);

var _attachParams = require("./middleware/attach-params");

var _attachParams2 = _interopRequireDefault(_attachParams);

var _serverError = require("./middleware/server-error");

var _serverError2 = _interopRequireDefault(_serverError);

var _router3 = require("./middleware/router");

var _router4 = _interopRequireDefault(_router3);

var _reloaderInjector = require("./middleware/reloader-injector");

var _reloaderInjector2 = _interopRequireDefault(_reloaderInjector);

var _ioWatchAssets = require("./middleware/io-watch-assets");

var _ioWatchAssets2 = _interopRequireDefault(_ioWatchAssets);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Server = (function () {
    /**
     * @private
     * @property {express.Application} _express
     */
    // _express = null;

    /**
     * @private
     * @property {express.Application} _swapper
     */
    // _swapper = null;

    /**
     * @property {socketio.Server} _sockets
     */
    // _sockets = null;

    /**
     * @private
     * @property {http.Server|https.Server} _server
     */
    // _server = null;

    /**
     * @private
     * @property {Router} router
     */
    // router = null;

    /**
     * @class Server
     * @constructor
     * @param {Object} options
     * @param {ModuleSwapper} options.swapper
     * @param {Logger} options.logger
     * @param {Object} options.routes
     * @param {Logger} options.logger
     */

    function Server(options) {
        _classCallCheck(this, Server);

        this._swapper = options.swapper;
        this._logger = options.logger;
        this._koa = (0, _koa3.default)();
        this._sockets = (0, _socket2.default)();
        this.router = new _router2.default(this._swapper, { logger: this._logger });

        this.use((0, _serverError2.default)());
    }

    /**
     * get socket.io host.
     * @return {Socketio.Server}
     */

    _createClass(Server, [{
        key: "getSocketIo",
        value: function getSocketIo() {
            return this._sockets;
        }

        /**
         * Start up http(s) server
         * @param {Object} options
         * @param {ConfigLoader} options.config
         * @param {String} options.appRoot Application running path
         * @param {Object|Boolean} options.https HTTPS options
         * @param {String} options.https.key path to SSL key
         * @param {String} options.https.cert path to SSL cert
         * @param {Number} options.port Listening port number
         * @param {Object} options.routes route definition object
         * @param {Boolean} options.watch if true, enable watch for Controller, Model, View
         */

    }, {
        key: "start",
        value: (function () {
            var ref = _asyncToGenerator(function* (options) {
                try {
                    this._registerMiddlewares(options);
                    this.router.load({
                        routes: options.routes,
                        controllerDir: _path2.default.join(options.appRoot, "controller/")
                    });
                    yield this._listen(options);
                } catch (e) {
                    this._logger.error("Server#start", e.message + "\n" + e.stack);
                    throw e;
                }
            });

            return function start(_x) {
                return ref.apply(this, arguments);
            };
        })()
    }, {
        key: "_registerMiddlewares",
        value: function _registerMiddlewares(options) {
            var controllersDir = _path2.default.join(options.appRoot, "controller/");
            var viewsDir = _path2.default.join(options.appRoot, "views/");

            this.use((0, _attachParams2.default)(this));
            this.use((0, _router4.default)(this.router));
        }
    }, {
        key: "_listen",
        value: (function () {
            var ref = _asyncToGenerator(function* (options) {
                var _this = this;

                // Get request handlers
                var handler = this._koa.callback();

                // Create server
                if (options.https) {
                    this._server = _https2.default.createServer({
                        key: _fs2.default.readFileSync(options.https.key),
                        cert: _fs2.default.readFileSync(options.https.cert)
                    }, handler);
                } else {
                    this._server = _http2.default.createServer(handler);
                }

                // Attach socket.io to server
                this._sockets.attach(this._server);

                // Start server
                return new Promise(function (resolve, reject) {
                    _this._server.listen(options.port, function (err) {
                        if (err) reject(err);
                        resolve();
                    });
                });
            });

            return function _listen(_x2) {
                return ref.apply(this, arguments);
            };
        })()
    }, {
        key: "use",
        value: function use() {
            var _koa;

            (_koa = this._koa).use.apply(_koa, arguments);
        }
    }]);

    return Server;
})();

exports.default = Server;